@@ -1,13 +1,11 @@
 from __future__ import annotations
 
 import os
-from pathlib import Path
 from typing import Any, Mapping
 
 from pyproject_hooks import HookMissing
 
 from pdm.builders.base import EnvBuilder
-from pdm.models.environment import Environment
 from pdm.termui import logger
 
 
@@ -19,12 +17,6 @@ class EditableBuilder(EnvBuilder):
         "requires": ["setuptools_pep660"],
     }
 
-    def __init__(self, src_dir: str | Path, environment: Environment) -> None:
-        super().__init__(src_dir, environment)
-        if self._hook.build_backend.startswith("pdm.pep517") and environment.interpreter.version_tuple < (3, 6):
-            # pdm.pep517 backend is not available on Python 2, use the fallback backend
-            self.init_build_system(self.FALLBACK_BACKEND)
-
     def prepare_metadata(self, out_dir: str, config_settings: Mapping[str, Any] | None = None) -> str:
         self.install(self._requires, shared=True)
         try:
