@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 import argparse
+from pathlib import Path
+from typing import TYPE_CHECKING
 
 from pdm import termui
 from pdm.cli import actions
@@ -9,17 +11,84 @@ from pdm.cli.hooks import HookManager
 from pdm.cli.options import skip_option
 from pdm.models.backends import _BACKENDS, DEFAULT_BACKEND, BuildBackend, get_backend
 from pdm.models.python import PythonInfo
-from pdm.project import Project
-from pdm.utils import get_user_email_from_git, get_venv_like_prefix
+from pdm.models.specifiers import get_specifier
+from pdm.models.venv import get_venv_python
+from pdm.utils import get_user_email_from_git
+
+if TYPE_CHECKING:
+    from pdm.project import Project
 
 
 class Command(BaseCommand):
     """Initialize a pyproject.toml for PDM"""
 
-    def __init__(self, parser: argparse.ArgumentParser) -> None:
-        super().__init__(parser)
+    def __init__(self) -> None:
         self.interactive = True
 
+    @staticmethod
+    def do_init(
+        project: Project,
+        name: str = "",
+        version: str = "",
+        description: str = "",
+        license: str = "MIT",
+        author: str = "",
+        email: str = "",
+        python_requires: str = "",
+        build_backend: type[BuildBackend] | None = None,
+        hooks: HookManager | None = None,
+    ) -> None:
+        """Bootstrap the project and create a pyproject.toml"""
+        from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
+
+        hooks = hooks or HookManager(project)
+        data = {
+            "project": {
+                "name": name,
+                "version": version,
+                "description": description,
+                "authors": array_of_inline_tables([{"name": author, "email": email}]),
+                "license": make_inline_table({"text": license}),
+                "dependencies": make_array([], True),
+            },
+        }
+        if build_backend is not None:
+            data["build-system"] = build_backend.build_system()
+        if python_requires and python_requires != "*":
+            data["project"]["requires-python"] = python_requires
+        if name and version:
+            readme = next(project.root.glob("README*"), None)
+            if readme is None:
+                readme = project.root.joinpath("README.md")
+                readme.write_text(f"# {name}\n\n{description}\n", encoding="utf-8")
+            data["project"]["readme"] = readme.name
+        get_specifier(python_requires)
+        project.pyproject._data.update(data)
+        project.pyproject.write()
+        Command._write_gitignore(project.root.joinpath(".gitignore"))
+        hooks.try_emit("post_init")
+
+    @staticmethod
+    def _write_gitignore(path: Path) -> None:
+        import requests
+
+        url = "https://raw.githubusercontent.com/github/gitignore/master/Python.gitignore"
+        if not path.exists():
+            try:
+                resp = requests.get(url, timeout=5)
+                resp.raise_for_status()
+            except requests.exceptions.RequestException:
+                content = "\n".join(["build/", "dist/", "*.egg-info/", "__pycache__/", "*.py[cod]"]) + "\n"
+            else:
+                content = resp.text
+            content += ".pdm-python\n"
+        else:
+            content = path.read_text(encoding="utf-8")
+            if ".pdm-python" in content:
+                return
+            content += ".pdm-python\n"
+        path.write_text(content, encoding="utf-8")
+
     def set_interactive(self, value: bool) -> None:
         self.interactive = value
 
@@ -42,7 +111,7 @@ class Command(BaseCommand):
         parser.set_defaults(search_parent=False)
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
-        from pdm.cli.commands.venv.utils import get_venv_python
+        from pdm.cli.commands.use import Command as UseCommand
 
         hooks = HookManager(project, options.skip)
         if project.pyproject.exists():
@@ -50,32 +119,19 @@ class Command(BaseCommand):
         else:
             project.core.ui.echo("Creating a pyproject.toml for PDM...", style="primary")
         self.set_interactive(not options.non_interactive)
-
+        do_use = UseCommand().do_use
         if self.interactive:
-            python = actions.do_use(
+            python = do_use(
                 project,
                 options.python or "",
                 first=bool(options.python),
                 ignore_remembered=True,
                 ignore_requires_python=True,
+                save=False,
                 hooks=hooks,
             )
-            if project.config["python.use_venv"] and get_venv_like_prefix(python.executable) is None:
-                if termui.confirm(
-                    f"Would you like to create a virtualenv with [success]{python.executable}[/]?",
-                    default=True,
-                ):
-                    try:
-                        path = project._create_virtualenv()
-                        python = project.python = PythonInfo.from_path(get_venv_python(path))
-                    except Exception as e:  # pragma: no cover
-                        project.core.ui.echo(
-                            f"Error occurred when creating virtualenv: {e}\nPlease fix it and create later.",
-                            style="error",
-                            err=True,
-                        )
         else:
-            python = actions.do_use(
+            python = do_use(
                 project,
                 options.python or "3",
                 first=True,
@@ -84,7 +140,22 @@ class Command(BaseCommand):
                 save=False,
                 hooks=hooks,
             )
-        if get_venv_like_prefix(python.executable) is None:
+        if project.config["python.use_venv"] and python.get_venv() is None:
+            if not self.interactive or termui.confirm(
+                f"Would you like to create a virtualenv with [success]{python.executable}[/]?",
+                default=True,
+            ):
+                project._python = python
+                try:
+                    path = project._create_virtualenv()
+                    python = PythonInfo.from_path(get_venv_python(path))
+                except Exception as e:  # pragma: no cover
+                    project.core.ui.echo(
+                        f"Error occurred when creating virtualenv: {e}\nPlease fix it and create later.",
+                        style="error",
+                        err=True,
+                    )
+        if python.get_venv() is None:
             project.core.ui.echo(
                 "You are using the PEP 582 mode, no virtualenv is created.\n"
                 "For more info, please visit https://peps.python.org/pep-0582/",
@@ -128,8 +199,9 @@ class Command(BaseCommand):
         email = self.ask("Author email", git_email)
         python_version = f"{python.major}.{python.minor}"
         python_requires = self.ask("Python requires('*' to allow any)", f">={python_version}")
+        project.python = python
 
-        actions.do_init(
+        self.do_init(
             project,
             name=name,
             version=version,
