@@ -9,29 +9,31 @@ import signal
 import subprocess
 import sys
 from types import FrameType
-from typing import Any, Callable, Iterator, Mapping, NamedTuple, Sequence, cast
+from typing import TYPE_CHECKING, Mapping, NamedTuple, Sequence, cast
+
+from rich import print_json
 
 from pdm import termui
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.hooks import HookManager
 from pdm.cli.options import skip_option, venv_option
 from pdm.cli.utils import check_project_file, get_pep582_path
-from pdm.compat import TypedDict
 from pdm.exceptions import PdmUsageError
 from pdm.project import Project
 from pdm.signals import pdm_signals
 from pdm.utils import is_path_relative_to
 
+if TYPE_CHECKING:
+    from typing import Any, Callable, Iterator, TypedDict
 
-class EnvFileOptions(TypedDict, total=True):
-    override: str
+    class EnvFileOptions(TypedDict, total=True):
+        override: str
 
-
-class TaskOptions(TypedDict, total=False):
-    env: Mapping[str, str]
-    env_file: EnvFileOptions | str | None
-    help: str
-    site_packages: bool
+    class TaskOptions(TypedDict, total=False):
+        env: Mapping[str, str]
+        env_file: EnvFileOptions | str | None
+        help: str
+        site_packages: bool
 
 
 def exec_opts(*options: TaskOptions | None) -> dict[str, Any]:
@@ -86,7 +88,7 @@ class TaskRunner:
     def __init__(self, project: Project, hooks: HookManager) -> None:
         self.project = project
         global_options = cast(
-            TaskOptions,
+            "TaskOptions",
             self.project.scripts.get("_", {}) if self.project.scripts else {},
         )
         self.global_options = global_options.copy()
@@ -114,7 +116,7 @@ class TaskRunner:
         unknown_options = set(options) - set(self.OPTIONS)
         if unknown_options:
             raise PdmUsageError(f"Unknown options for task {script_name}: {', '.join(unknown_options)}")
-        return Task(kind, script_name, value, cast(TaskOptions, options))
+        return Task(kind, script_name, value, cast("TaskOptions", options))
 
     def _run_process(
         self,
@@ -232,13 +234,14 @@ class TaskRunner:
             assert isinstance(value, list)
 
         self.project.core.ui.echo(
-            f"Running {task}: [success]{str(args)}[/]",
+            f"Running {task}: [success]{args}[/]",
             err=True,
             verbosity=termui.Verbosity.DETAIL,
         )
         if kind == "composite":
             args = list(args)
             should_interpolate = any(RE_ARGS_PLACEHOLDER.search(script) for script in value)
+            code = 0
             for script in value:
                 if should_interpolate:
                     script, _ = interpolate(script, args)
@@ -287,7 +290,7 @@ class TaskRunner:
         columns = ["Name", "Type", "Description"]
         result = []
         for name in sorted(self.project.scripts):
-            if name == "_":
+            if name.startswith("_"):
                 continue
             task = self.get_task(name)
             assert task is not None
@@ -300,6 +303,33 @@ class TaskRunner:
             )
         self.project.core.ui.display_columns(result, columns)
 
+    def as_json(self) -> dict[str, Any]:
+        out = {}
+        for name in sorted(self.project.scripts):
+            if name == "_":
+                data = out["_"] = dict(name="_", kind="shared", help="Shared options", **self.global_options)
+                _fix_env_file(data)
+                continue
+            task = self.get_task(name)
+            assert task is not None
+            data = out[name] = {
+                "name": name,
+                "kind": task.kind,
+                "help": task.short_description,
+                "args": task.args,  # type: ignore[dict-item]
+            }
+            data.update(**task.options)
+            _fix_env_file(data)
+        return out
+
+
+def _fix_env_file(data: dict[str, Any]) -> dict[str, Any]:
+    env_file = data.get("env_file")
+    if isinstance(env_file, dict):
+        del data["env_file"]
+        data["env_file.override"] = env_file.get("override")
+    return data
+
 
 class Command(BaseCommand):
     """Run commands or scripts with local packages loaded"""
@@ -308,20 +338,28 @@ class Command(BaseCommand):
     arguments = [*BaseCommand.arguments, skip_option, venv_option]
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
-        parser.add_argument(
+        action = parser.add_mutually_exclusive_group()
+        action.add_argument(
             "-l",
             "--list",
             action="store_true",
             help="Show all available scripts defined in pyproject.toml",
         )
-        parser.add_argument(
+        action.add_argument(
+            "-j",
+            "--json",
+            action="store_true",
+            help="Output all scripts infos in JSON",
+        )
+        exec = parser.add_argument_group("Execution parameters")
+        exec.add_argument(
             "-s",
             "--site-packages",
             action="store_true",
             help="Load site-packages from the selected interpreter",
         )
-        parser.add_argument("script", nargs="?", help="The command to run")
-        parser.add_argument(
+        exec.add_argument("script", nargs="?", help="The command to run")
+        exec.add_argument(
             "args",
             nargs=argparse.REMAINDER,
             help="Arguments that will be passed to the command",
@@ -333,6 +371,8 @@ class Command(BaseCommand):
         runner = self.runner_cls(project, hooks=hooks)
         if options.list:
             return runner.show_list()
+        if options.json:
+            return print_json(data=runner.as_json())
         if options.site_packages:
             runner.global_options.update({"site_packages": options.site_packages})
         if not options.script:
