@@ -64,7 +64,7 @@ class Command(BaseCommand):
         return super().register_to(subparsers, name, aliases=["plugin"], **kwargs)
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
-        subparsers = parser.add_subparsers(title="Sub commands")
+        subparsers = parser.add_subparsers(title="commands", metavar="self")
         ListCommand.register_to(subparsers)
         if not is_in_zipapp():
             AddCommand.register_to(subparsers)
@@ -237,7 +237,7 @@ class UpdateCommand(BaseCommand):
                 project.core.ui.echo(f"Already up-to-date: [primary]{__version__}[/]")
                 return
             package = f"pdm=={version}"
-        pip_args = ["install", "--upgrade", *shlex.split(options.pip_args)] + [package]
+        pip_args = ["install", "--upgrade", *shlex.split(options.pip_args), package]
         project.core.ui.echo(f"Running pip command: {pip_args}", verbosity=termui.Verbosity.DETAIL)
         try:
             with project.core.ui.open_spinner(f"Updating pdm to version [primary]{version}[/]"):
