@@ -4,7 +4,7 @@ from pathlib import Path
 
 from pdm import termui
 from pdm.cli.commands.base import BaseCommand
-from pdm.cli.commands.venv.utils import get_venv_with_name, iter_venvs
+from pdm.cli.commands.venv.utils import get_venv_with_name
 from pdm.cli.options import verbose_option
 from pdm.project import Project
 
@@ -26,9 +26,9 @@ class RemoveCommand(BaseCommand):
     def handle(self, project: Project, options: argparse.Namespace) -> None:
         project.core.ui.echo("Virtualenvs created with this project:")
         venv = get_venv_with_name(project, options.env)
-        if options.yes or termui.confirm(f"[warning]Will remove: [success]{venv}[/], continue?", default=True):
-            shutil.rmtree(venv)
+        if options.yes or termui.confirm(f"[warning]Will remove: [success]{venv.root}[/], continue?", default=True):
+            shutil.rmtree(venv.root)
             saved_python = project._saved_python
-            if saved_python and Path(saved_python).parent.parent == venv:
+            if saved_python and Path(saved_python).parent.parent == venv.root:
                 project._saved_python = None
             project.core.ui.echo("Removed successfully!")
