@@ -2,31 +2,28 @@ from __future__ import annotations
 
 import base64
 import hashlib
-import sys
 from pathlib import Path
 from typing import Iterable, TypeVar
+
 from findpython import PythonVersion
-
 from findpython.providers import BaseProvider
+
 from pdm.exceptions import PdmUsageError
-
+from pdm.models.venv import VirtualEnv
 from pdm.project import Project
 
-IS_WIN = sys.platform == "win32"
-BIN_DIR = "Scripts" if IS_WIN else "bin"
-
 
 def hash_path(path: str) -> str:
     """Generate a hash for the given path."""
     return base64.urlsafe_b64encode(hashlib.new("md5", path.encode(), usedforsecurity=False).digest()).decode()[:8]
 
 
-def get_in_project_venv_python(root: Path) -> Path | None:
+def get_in_project_venv(root: Path) -> VirtualEnv | None:
     """Get the python interpreter path of venv-in-project"""
     for possible_dir in (".venv", "venv", "env"):
-        venv_python = get_venv_python(root / possible_dir)
-        if venv_python.exists():
-            return venv_python
+        venv = VirtualEnv.get(root / possible_dir)
+        if venv is not None:
+            return venv
     return None
 
 
@@ -37,22 +34,18 @@ def get_venv_prefix(project: Project) -> str:
     return f"{path.name}-{name_hash}-"
 
 
-def iter_venvs(project: Project) -> Iterable[tuple[str, Path]]:
+def iter_venvs(project: Project) -> Iterable[tuple[str, VirtualEnv]]:
     """Return an iterable of venv paths associated with the project"""
-    in_project_venv_python = get_in_project_venv_python(project.root)
-    if in_project_venv_python is not None:
-        yield "in-project", Path(in_project_venv_python).parent.parent
+    in_project_venv = get_in_project_venv(project.root)
+    if in_project_venv is not None:
+        yield "in-project", in_project_venv
     venv_prefix = get_venv_prefix(project)
     venv_parent = Path(project.config["venv.location"])
-    for venv in venv_parent.glob(f"{venv_prefix}*"):
-        ident = venv.name[len(venv_prefix) :]
-        yield ident, venv
-
-
-def get_venv_python(venv: Path) -> Path:
-    """Get the interpreter path inside the given venv."""
-    suffix = ".exe" if IS_WIN else ""
-    return venv / BIN_DIR / f"python{suffix}"
+    for path in venv_parent.glob(f"{venv_prefix}*"):
+        ident = path.name[len(venv_prefix) :]
+        venv = VirtualEnv.get(path)
+        if venv is not None:
+            yield ident, venv
 
 
 def iter_central_venvs(project: Project) -> Iterable[tuple[str, Path]]:
@@ -78,12 +71,10 @@ class VenvProvider(BaseProvider):
 
     def find_pythons(self) -> Iterable[PythonVersion]:
         for _, venv in iter_venvs(self.project):
-            python = get_venv_python(venv)
-            if python.exists():
-                yield PythonVersion(python, _interpreter=python, keep_symlink=True)
+            yield PythonVersion(venv.interpreter, _interpreter=venv.interpreter, keep_symlink=True)
 
 
-def get_venv_with_name(project: Project, name: str) -> Path:
+def get_venv_with_name(project: Project, name: str) -> VirtualEnv:
     all_venvs = dict(iter_venvs(project))
     try:
         return all_venvs[name]
@@ -91,4 +82,4 @@ def get_venv_with_name(project: Project, name: str) -> Path:
         raise PdmUsageError(
             f"No virtualenv with key '{name}' is found, must be one of {list(all_venvs)}.\n"
             "You can create one with 'pdm venv create'.",
-        )
+        ) from None
