@@ -4,16 +4,16 @@ import argparse
 import os
 import sys
 from functools import partial
-from typing import Any, Sequence, cast
+from typing import TYPE_CHECKING, cast
 
-from pdm.cli.actions import print_pep582_command
-from pdm.compat import Protocol
-from pdm.project import Project
+if TYPE_CHECKING:
+    from typing import Any, Protocol, Sequence
 
+    from pdm.project import Project
 
-class ActionCallback(Protocol):
-    def __call__(self, project: Project, namespace: argparse.Namespace, values: str | Sequence[Any] | None) -> None:
-        ...
+    class ActionCallback(Protocol):
+        def __call__(self, project: Project, namespace: argparse.Namespace, values: str | Sequence[Any] | None) -> None:
+            ...
 
 
 class Option:
@@ -127,7 +127,7 @@ verbose_option = Option(
     "--verbose",
     action="count",
     default=0,
-    help="-v for detailed output and -vv for more detailed",
+    help="Use `-v` for detailed output and `-vv` for more detailed",
 )
 
 
@@ -149,6 +149,8 @@ lockfile_option = Option(
 
 @Option("--pep582", const="AUTO", metavar="SHELL", nargs="?", help="Print the command line to be eval'd by the shell")
 def pep582_option(project: Project, namespace: argparse.Namespace, values: str | Sequence[Any] | None) -> None:
+    from pdm.cli.actions import print_pep582_command
+
     print_pep582_command(project, cast(str, values))
     sys.exit(0)
 
@@ -163,8 +165,9 @@ install_group.add_argument(
 install_group.add_argument(
     "--no-self",
     action="store_true",
+    default=bool(os.getenv("PDM_NO_SELF")),
     dest="no_self",
-    help="Don't install the project itself",
+    help="Don't install the project itself. [env var: PDM_NO_SELF]",
 )
 install_group.add_argument("--fail-fast", "-x", action="store_true", help="Abort on first installation error")
 
@@ -184,7 +187,7 @@ groups_group.add_argument(
     metavar="GROUP",
     action=split_lists(","),
     help="Select group of optional-dependencies separated by comma "
-    "or dev-dependencies(with -d). Can be supplied multiple times, "
+    "or dev-dependencies (with `-d`). Can be supplied multiple times, "
     'use ":all" to include all groups under the same species.',
     default=[],
 )
@@ -296,12 +299,12 @@ global_option = Option(
 )
 
 clean_group = ArgumentGroup("clean", is_mutually_exclusive=True)
-clean_group.add_argument("--clean", action="store_true", help="clean packages not in the lockfile")
-clean_group.add_argument("--only-keep", action="store_true", help="only keep the selected packages")
+clean_group.add_argument("--clean", action="store_true", help="Clean packages not in the lockfile")
+clean_group.add_argument("--only-keep", action="store_true", help="Only keep the selected packages")
 
 sync_group = ArgumentGroup("sync", is_mutually_exclusive=True)
-sync_group.add_argument("--sync", action="store_true", dest="sync", help="sync packages")
-sync_group.add_argument("--no-sync", action="store_false", dest="sync", help="don't sync packages")
+sync_group.add_argument("--sync", action="store_true", dest="sync", help="Sync packages")
+sync_group.add_argument("--no-sync", action="store_false", dest="sync", help="Don't sync packages")
 
 packages_group = ArgumentGroup("Package Arguments")
 packages_group.add_argument(
