@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import argparse
+import dataclasses as dc
 import json
 import os
 import sys
@@ -26,8 +27,6 @@ from rich.tree import Tree
 
 from pdm import termui
 from pdm.exceptions import PdmArgumentError, ProjectError
-from pdm.formats import FORMATS
-from pdm.formats.base import make_array, make_inline_table
 from pdm.models.requirements import (
     Requirement,
     filter_requirements_with_extras,
@@ -55,23 +54,6 @@ if TYPE_CHECKING:
     from pdm.project import Project
 
 
-class ErrorArgumentParser(argparse.ArgumentParser):
-    """A subclass of argparse.ArgumentParser that raises
-    parsing error rather than exiting.
-
-    This does the same as passing exit_on_error=False on Python 3.9+
-    """
-
-    def _parse_known_args(
-        self, arg_strings: list[str], namespace: argparse.Namespace
-    ) -> tuple[argparse.Namespace, list[str]]:
-        try:
-            return super()._parse_known_args(arg_strings, namespace)
-        except argparse.ArgumentError as e:
-            # We raise a dedicated error to avoid being caught by the caller
-            raise PdmArgumentError(e) from e
-
-
 class PdmFormatter(argparse.RawDescriptionHelpFormatter):
     def start_section(self, heading: str | None) -> None:
         return super().start_section(termui.style(heading.title() if heading else "", style="warning"))
@@ -137,25 +119,46 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
         return self._join_parts(parts)
 
 
+class ArgumentParser(argparse.ArgumentParser):
+    """A standard argument parser but with title-cased help."""
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        kwargs["formatter_class"] = PdmFormatter
+        kwargs["add_help"] = False
+        super().__init__(*args, **kwargs)
+        self.add_argument(
+            "-h", "--help", action="help", default=argparse.SUPPRESS, help="Show this help message and exit."
+        )
+
+
+class ErrorArgumentParser(ArgumentParser):
+    """A subclass of argparse.ArgumentParser that raises
+    parsing error rather than exiting.
+
+    This does the same as passing exit_on_error=False on Python 3.9+
+    """
+
+    def _parse_known_args(
+        self, arg_strings: list[str], namespace: argparse.Namespace
+    ) -> tuple[argparse.Namespace, list[str]]:
+        try:
+            return super()._parse_known_args(arg_strings, namespace)
+        except argparse.ArgumentError as e:
+            # We raise a dedicated error to avoid being caught by the caller
+            raise PdmArgumentError(e) from e
+
+
+@dc.dataclass(frozen=True)
 class Package:
     """An internal class for the convenience of dependency graph building."""
 
-    def __init__(self, name: str, version: str | None, requirements: dict[str, Requirement]) -> None:
-        self.name = name
-        self.version = version  # if version is None, the dist is not installed.
-        self.requirements = requirements
-
-    def __hash__(self) -> int:
-        return hash(self.name)
+    name: str = dc.field(hash=True, compare=True)
+    version: str | None = dc.field(compare=False)
+    requirements: dict[str, Requirement] = dc.field(compare=False)
 
     def __repr__(self) -> str:
         return f"<Package {self.name}=={self.version}>"
 
-    def __eq__(self, value: object) -> bool:
-        if not isinstance(value, Package):
-            return False
-        return self.name == value.name
-
 
 def build_dependency_graph(
     working_set: Mapping[str, im.Distribution],
@@ -434,6 +437,7 @@ def format_lockfile(
     """Format lock file from a dict of resolved candidates, a mapping of dependencies
     and a collection of package summaries.
     """
+    from pdm.formats.base import make_array, make_inline_table
 
     packages = tomlkit.aot()
     file_hashes = tomlkit.table()
@@ -504,6 +508,8 @@ def check_project_file(project: Project) -> None:
 
 def find_importable_files(project: Project) -> Iterable[tuple[str, Path]]:
     """Find all possible files that can be imported"""
+    from pdm.formats import FORMATS
+
     for filename in (
         "Pipfile",
         "pyproject.toml",
@@ -561,7 +567,7 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
         ]
         for req, parent in conflicting:
             pyspec &= req.specifier
-            info_lines.add(f"  {req.as_line()} (from {repr(parent)})")
+            info_lines.add(f"  {req.as_line()} (from {parent!r})")
         result.extend(sorted(info_lines))
         if pyspec.is_impossible:
             result.append("Consider changing the version specifiers of the dependencies to be compatible")
@@ -604,9 +610,10 @@ def merge_dictionary(target: MutableMapping[Any, Any], input: Mapping[Any, Any])
         if key not in target:
             target[key] = value
         elif isinstance(value, dict):
-            target[key].update(value)
+            merge_dictionary(target[key], value)
         elif isinstance(value, list):
-            target[key].extend([x for x in value if x not in target[key]])
+            target[key].extend(x for x in value if x not in target[key])
+            target[key].multiline(True)  # type: ignore[attr-defined]
         else:
             target[key] = value
 
@@ -663,11 +670,19 @@ def get_pep582_path(project: Project) -> str:
 
 
 def use_venv(project: Project, name: str) -> None:
-    from pdm.cli.commands.venv.utils import get_venv_python, get_venv_with_name
+    from pdm.cli.commands.venv.utils import get_venv_with_name
     from pdm.environments import PythonEnvironment
 
     venv = get_venv_with_name(project, cast(str, name))
     project.core.ui.echo(
-        f"In virtual environment: [success]{venv}[/]", err=True, style="info", verbosity=termui.Verbosity.DETAIL
+        f"In virtual environment: [success]{venv.root}[/]", err=True, style="info", verbosity=termui.Verbosity.DETAIL
     )
-    project.environment = PythonEnvironment(project, python=str(get_venv_python(venv)))
+    project.environment = PythonEnvironment(project, python=str(venv.interpreter))
+
+
+def populate_requirement_names(req_mapping: dict[str, Requirement]) -> None:
+    # Update the requirement key if the name changed.
+    for key, req in list(req_mapping.items()):
+        if key and key.startswith(":empty:"):
+            req_mapping[req.identify()] = req
+            del req_mapping[key]
