@@ -14,24 +14,28 @@ import os
 import pkgutil
 import sys
 from pathlib import Path
-from typing import Any, Iterable, cast
+from typing import TYPE_CHECKING, cast
 
 from resolvelib import Resolver
 
 from pdm import termui
 from pdm.__version__ import __version__
-from pdm.cli.actions import check_update, print_pep582_command
-from pdm.cli.commands.base import BaseCommand
 from pdm.cli.options import ignore_python_option, pep582_option, verbose_option
-from pdm.cli.utils import ErrorArgumentParser, PdmFormatter
+from pdm.cli.utils import ArgumentParser, ErrorArgumentParser
 from pdm.compat import importlib_metadata
 from pdm.exceptions import PdmArgumentError, PdmUsageError
 from pdm.installers import InstallManager, Synchronizer
 from pdm.models.repositories import PyPIRepository
 from pdm.project import Project
-from pdm.project.config import Config, ConfigItem
+from pdm.project.config import Config
 from pdm.utils import is_in_zipapp
 
+if TYPE_CHECKING:
+    from typing import Any, Iterable
+
+    from pdm.cli.commands.base import BaseCommand
+    from pdm.project.config import ConfigItem
+
 COMMANDS_MODULE_PATH = importlib.import_module("pdm.cli.commands").__path__
 
 
@@ -57,7 +61,6 @@ class Core:
         self.parser = ErrorArgumentParser(
             prog="pdm",
             description=__doc__,
-            formatter_class=PdmFormatter,
         )
         self.parser.is_root = True  # type: ignore[attr-defined]
         self.parser.add_argument(
@@ -68,19 +71,19 @@ class Core:
                 termui.style("PDM", style="bold"),
                 termui.style(self.version, style="success"),
             ),
-            help="show the version and exit",
+            help="Show the version and exit",
         )
         self.parser.add_argument(
             "-c",
             "--config",
-            help="Specify another config file path(env var: PDM_CONFIG_FILE)",
+            help="Specify another config file path [env var: PDM_CONFIG_FILE] ",
         )
         self.parser._positionals.title = "Commands"
         verbose_option.add_to_parser(self.parser)
         ignore_python_option.add_to_parser(self.parser)
         pep582_option.add_to_parser(self.parser)
 
-        self.subparsers = self.parser.add_subparsers(parser_class=argparse.ArgumentParser)
+        self.subparsers = self.parser.add_subparsers(parser_class=ArgumentParser, metavar="__root__")
         for _, name, _ in pkgutil.iter_modules(COMMANDS_MODULE_PATH):
             module = importlib.import_module(f"pdm.cli.commands.{name}", __name__)
             try:
@@ -104,9 +107,6 @@ class Core:
                 is_global=global_project,
                 global_config=options.config or os.getenv("PDM_CONFIG_FILE"),
             )
-
-        if getattr(options, "lockfile", None):
-            project.set_lockfile(options.lockfile)
         return project
 
     def create_project(
@@ -127,6 +127,46 @@ class Core:
         """
         return self.project_class(self, root_path, is_global, global_config)
 
+    def handle(self, project: Project, options: argparse.Namespace) -> None:
+        """Called before command invocation"""
+        from pdm.cli.commands.fix import Command as FixCommand
+        from pdm.cli.hooks import HookManager
+        from pdm.cli.utils import use_venv
+
+        self.ui.set_verbosity(options.verbose)
+        self.ui.set_theme(project.global_config.load_theme())
+
+        command = cast("BaseCommand | None", getattr(options, "command", None))
+        hooks = HookManager(project, getattr(options, "skip", None))
+        hooks.try_emit("pre_invoke", command=command.name if command else None, options=options)
+
+        if not isinstance(command, FixCommand):
+            FixCommand.check_problems(project)
+
+        for callback in getattr(options, "callbacks", []):
+            callback(project, options)
+
+        if getattr(options, "lockfile", None):
+            project.set_lockfile(cast(str, options.lockfile))
+
+        if getattr(options, "use_venv", None):
+            use_venv(project, cast(str, options.use_venv))
+
+        if command is None:
+            self.parser.error("No command given")
+        command.handle(project, options)
+
+    def _get_cli_args(self, args: list[str], obj: Project | None) -> list[str]:
+        project = self.create_project(is_global=False) if obj is None else obj
+        if project.is_global:
+            return args
+        config = project.pyproject.settings.get("options", {})
+        (pos, command) = next(((i, arg) for i, arg in enumerate(args) if not arg.startswith("-")), (-1, None))
+        if command and command in config:
+            # add args after the command
+            args[pos + 1 : pos + 1] = list(config[command])
+        return args
+
     def main(
         self,
         args: list[str] | None = None,
@@ -135,8 +175,9 @@ class Core:
         **extra: Any,
     ) -> None:
         """The main entry function"""
+
         # Ensure same behavior while testing and using the CLI
-        args = args or sys.argv[1:]
+        args = self._get_cli_args(args or sys.argv[1:], obj)
         # Keep it for after project parsing to check if its a defined script
         root_script = None
         try:
@@ -153,46 +194,29 @@ class Core:
             except PdmArgumentError as e:
                 self.parser.error(str(e.__cause__))
 
-        self.ui.set_verbosity(options.verbose)
         project = self.ensure_project(options, obj)
-        self.ui.set_theme(project.global_config.load_theme())
-        if options.ignore_python:
-            os.environ["PDM_IGNORE_SAVED_PYTHON"] = "1"
-
-        if options.pep582:
-            print_pep582_command(project, options.pep582)
-            sys.exit(0)
-
         if root_script and root_script not in project.scripts:
-            self.parser.error(f"Command unknown: {root_script}")
+            self.parser.error(f"Script unknown: {root_script}")
 
         try:
-            f = options.handler
-        except AttributeError:
-            self.parser.print_help(sys.stderr)
+            self.handle(project, options)
+        except Exception:
+            etype, err, traceback = sys.exc_info()
+            should_show_tb = not isinstance(err, PdmUsageError)
+            if self.ui.verbosity > termui.Verbosity.NORMAL and should_show_tb:
+                raise cast(Exception, err).with_traceback(traceback) from None
+            self.ui.echo(
+                rf"[error]\[{etype.__name__}][/]: {err}",  # type: ignore[union-attr]
+                err=True,
+            )
+            if should_show_tb:
+                self.ui.echo("Add '-v' to see the detailed traceback", style="warning", err=True)
             sys.exit(1)
         else:
-            try:
-                f(project, options)
-            except Exception:
-                etype, err, traceback = sys.exc_info()
-                should_show_tb = not isinstance(err, PdmUsageError)
-                if self.ui.verbosity > termui.Verbosity.NORMAL and should_show_tb:
-                    raise cast(Exception, err).with_traceback(traceback) from None
-                self.ui.echo(
-                    rf"[error]\[{etype.__name__}][/]: {err}",  # type: ignore[union-attr]
-                    err=True,
-                )
-                if should_show_tb:
-                    self.ui.echo(
-                        "Add '-v' to see the detailed traceback",
-                        style="warning",
-                        err=True,
-                    )
-                sys.exit(1)
-            else:
-                if project.config["check_update"] and not is_in_zipapp():
-                    check_update(project)
+            if project.config["check_update"] and not is_in_zipapp():
+                from pdm.cli.actions import check_update
+
+                check_update(project)
 
     def register_command(self, command: type[BaseCommand], name: str | None = None) -> None:
         """Register a subcommand to the subparsers,
@@ -217,6 +241,23 @@ class Core:
         """
         Config.add_config(name, config_item)
 
+    def _add_project_plugins_library(self) -> None:
+        project = self.create_project(is_global=False)
+        if project.is_global or not project.root.joinpath(".pdm-plugins").exists():
+            return
+
+        import site
+        import sysconfig
+
+        base = str(project.root / ".pdm-plugins")
+        replace_vars = {"base": base, "platbase": base}
+        scheme = "nt" if os.name == "nt" else "posix_prefix"
+        purelib = sysconfig.get_path("purelib", scheme, replace_vars)
+        scripts = sysconfig.get_path("scripts", scheme, replace_vars)
+        site.addsitedir(purelib)
+        if os.path.exists(scripts):
+            os.environ["PATH"] = os.pathsep.join([scripts, os.getenv("PATH", "")])
+
     def load_plugins(self) -> None:
         """Import and load plugins under `pdm.plugin` namespace
         A plugin is a callable that accepts the core object as the only argument.
@@ -227,6 +268,7 @@ class Core:
                 ...
             ```
         """
+        self._add_project_plugins_library()
         entry_points: Iterable[importlib_metadata.EntryPoint] = itertools.chain(
             importlib_metadata.entry_points(group="pdm"),
             importlib_metadata.entry_points(group="pdm.plugin"),
