@@ -5,16 +5,19 @@ import dataclasses
 import hashlib
 import shlex
 import urllib.parse
-from argparse import Namespace
-from os import PathLike
-from typing import Any, Mapping
+from typing import TYPE_CHECKING, Any, Mapping
 
 from pdm.formats.base import make_array
-from pdm.models.candidates import Candidate
-from pdm.models.requirements import Requirement, parse_requirement
-from pdm.project import Project
+from pdm.models.requirements import FileRequirement, Requirement, parse_requirement
 from pdm.utils import expand_env_vars_in_auth
 
+if TYPE_CHECKING:
+    from argparse import Namespace
+    from os import PathLike
+
+    from pdm.models.candidates import Candidate
+    from pdm.project import Project
+
 
 class RequirementParser:
     """Reference:
@@ -51,7 +54,11 @@ class RequirementParser:
         if not line.startswith("-"):
             # Starts with a requirement, just ignore all per-requirement options
             req_string = line.split(" -", 1)[0].strip()
-            self.requirements.append(parse_requirement(req_string))
+            req = parse_requirement(req_string)
+            if not req.name:
+                assert isinstance(req, FileRequirement)
+                req.name = req.guess_name()
+            self.requirements.append(req)
             return
         args, _ = self._parser.parse_known_args(shlex.split(line))
         if args.index_url:
@@ -172,21 +179,25 @@ def export(
     candidates: list[Candidate] | list[Requirement],
     options: Namespace,
 ) -> str:
+    from pdm.models.candidates import Candidate
+
     lines = ["# This file is @generated by PDM.\n# Please do not edit it manually.\n\n"]
-    for candidate in sorted(candidates, key=lambda x: x.identify()):
+    for candidate in sorted(candidates, key=lambda x: x.identify()):  # type: ignore[attr-defined]
         if isinstance(candidate, Candidate):
             req = dataclasses.replace(candidate.req, specifier=f"=={candidate.version}", marker=None)
         else:
             assert isinstance(candidate, Requirement)
             req = candidate
-        lines.append(project.backend.expand_line(req.as_line()))
+        lines.append(project.backend.expand_line(req.as_line(), options.expandvars))
         if options.hashes and getattr(candidate, "hashes", None):
             for item in sorted(set(candidate.hashes.values())):  # type: ignore[attr-defined]
                 lines.append(f" \\\n    --hash={item}")
         lines.append("\n")
     sources = project.pyproject.settings.get("source", [])
     for source in sources:
-        url = expand_env_vars_in_auth(source["url"])
+        url = source["url"]
+        if options.expandvars:
+            url = expand_env_vars_in_auth(url)
         prefix = "--index-url" if source["name"] == "pypi" else "--extra-index-url"
         lines.append(f"{prefix} {url}\n")
         if not source.get("verify_ssl", True):
