@@ -80,7 +80,7 @@ def editables_candidate(environment: BaseEnvironment) -> Candidate | None:
     return next(iter(repository.find_candidates(parse_requirement("editables"))), None)
 
 
-class Synchronizer:
+class BaseSynchronizer:
     """Synchronize the working set with given installation candidates
 
     :param candidates: a dict of candidates to be installed
@@ -108,10 +108,10 @@ class Synchronizer:
         retry_times: int = 1,
         install_self: bool = False,
         no_editable: bool | Collection[str] = False,
-        use_install_cache: bool = False,
         reinstall: bool = False,
         only_keep: bool = False,
         fail_fast: bool = False,
+        use_install_cache: bool | None = None,
     ) -> None:
         self.requested_candidates = candidates
         self.environment = environment
@@ -120,6 +120,8 @@ class Synchronizer:
         self.retry_times = retry_times
         self.no_editable = no_editable
         self.install_self = install_self
+        if use_install_cache is None:
+            use_install_cache = environment.project.config["install.cache"]
         self.use_install_cache = use_install_cache
         self.reinstall = reinstall
         self.only_keep = only_keep
@@ -166,14 +168,6 @@ class Synchronizer:
         metadata = self.self_candidate.prepare(self.environment).metadata
         return any(req.startswith("editables") for req in metadata.requires or [])
 
-    def create_executor(
-        self,
-    ) -> ThreadPoolExecutor | DummyExecutor:
-        if self.parallel:
-            return ThreadPoolExecutor(max_workers=min(multiprocessing.cpu_count(), 8))
-        else:
-            return DummyExecutor()
-
     @property
     def manager(self) -> InstallManager:
         if not self._manager:
@@ -219,7 +213,7 @@ class Synchronizer:
         all_candidate_keys = list(locked_repository.all_candidates)
 
         for key, dist in working_set.items():
-            if key == self.self_key:
+            if key == self.self_key and self.install_self:
                 continue
             if key in candidates:
                 can = candidates.pop(key)
@@ -242,6 +236,37 @@ class Synchronizer:
             sorted(to_remove),
         )
 
+    def synchronize(self) -> None:
+        """Synchronize the working set with pinned candidates."""
+        to_add, to_update, to_remove = self.compare_with_working_set()
+        manager = self.manager
+        for key in to_add:
+            can = self.candidates[key]
+            termui.logger.info("Installing %s@%s...", key, can.version)
+            manager.install(can)
+        for key in to_update:
+            can = self.candidates[key]
+            dist = self.working_set[strip_extras(key)[0]]
+            dist_version = dist.version
+            termui.logger.info("Updating %s@%s -> %s...", key, dist_version, can.version)
+            manager.uninstall(dist)
+            manager.install(can)
+        for key in to_remove:
+            dist = self.working_set[key]
+            termui.logger.info("Removing %s@%s...", key, dist.version)
+            manager.uninstall(dist)
+        termui.logger.info("Synchronization complete.")
+
+
+class Synchronizer(BaseSynchronizer):
+    def create_executor(
+        self,
+    ) -> ThreadPoolExecutor | DummyExecutor:
+        if self.parallel:
+            return ThreadPoolExecutor(max_workers=min(multiprocessing.cpu_count(), 8))
+        else:
+            return DummyExecutor()
+
     def install_candidate(self, key: str, progress: Progress) -> Candidate:
         """Install candidate"""
         can = self.candidates[key]
@@ -349,7 +374,6 @@ class Synchronizer:
             self.ui.echo("All packages are synced to date, nothing to do.")
 
     def synchronize(self) -> None:
-        """Synchronize the working set with pinned candidates."""
         to_add, to_update, to_remove = self.compare_with_working_set()
         to_do = {"remove": to_remove, "update": to_update, "add": to_add}
 
