@@ -5,21 +5,22 @@ import dataclasses
 import hashlib
 import json
 import os
+from functools import lru_cache
 from pathlib import Path
 from typing import TYPE_CHECKING, BinaryIO, Generic, Iterable, TypeVar, cast
 
-import requests
-from cachecontrol.cache import BaseCache
+from cachecontrol.cache import SeparateBodyBaseCache
 from cachecontrol.caches import FileCache
 from packaging.utils import canonicalize_name, parse_wheel_filename
-
 from pdm._types import CandidateInfo
 from pdm.exceptions import PdmException
 from pdm.models.candidates import Candidate
 from pdm.termui import logger
-from pdm.utils import atomic_open_for_write
+from pdm.utils import atomic_open_for_write, create_tracked_tempdir
 
 if TYPE_CHECKING:
+    from packaging.tags import Tag
+    from requests import Session
     from unearth import Link, TargetPython
 
 KT = TypeVar("KT")
@@ -106,26 +107,28 @@ class HashCache:
     def __init__(self, directory: Path) -> None:
         self.directory = directory
 
-    def _read_from_link(self, link: Link, session: requests.Session) -> Iterable[bytes]:
+    def _read_from_link(self, link: Link, session: Session) -> Iterable[bytes]:
         if link.is_file:
             with open(link.file_path, "rb") as f:
                 yield from f
         else:
-            resp = session.get(link.normalized, stream=True)
-            try:
-                resp.raise_for_status()
-            except requests.HTTPError as e:
-                raise PdmException(f"Failed to read from {link.redacted}: {e}") from e
-            yield from resp.iter_content(chunk_size=8096)
+            import requests
 
-    def _get_file_hash(self, link: Link, session: requests.Session) -> str:
+            with session.get(link.normalized, stream=True) as resp:
+                try:
+                    resp.raise_for_status()
+                except requests.HTTPError as e:
+                    raise PdmException(f"Failed to read from {link.redacted}: {e}") from e
+                yield from resp.iter_content(chunk_size=8192)
+
+    def _get_file_hash(self, link: Link, session: Session) -> str:
         h = hashlib.new(self.FAVORITE_HASH)
         logger.debug("Downloading link %s for calculating hash", link.redacted)
         for chunk in self._read_from_link(link, session):
             h.update(chunk)
         return ":".join([h.name, h.hexdigest()])
 
-    def get_hash(self, link: Link, session: requests.Session) -> str:
+    def get_hash(self, link: Link, session: Session) -> str:
         # If there is no link hash (i.e., md5, sha256, etc.), we don't want
         # to store it.
         hash_value = self.get(link.url_without_fragment)
@@ -171,16 +174,16 @@ class WheelCache:
 
     def __init__(self, directory: Path) -> None:
         self.directory = directory
+        self.ephemeral_directory = Path(create_tracked_tempdir(prefix="pdm-wheel-cache-"))
 
-    def _get_candidates(self, link: Link, target_python: TargetPython) -> Iterable[Path]:
-        path = self.get_path_for_link(link, target_python)
+    def _get_candidates(self, path: Path) -> Iterable[Path]:
         if not path.exists():
             return
         for candidate in path.iterdir():
             if candidate.name.endswith(".whl"):
                 yield candidate
 
-    def get_path_for_link(self, link: Link, target_python: TargetPython) -> Path:
+    def _get_path_parts(self, link: Link, target_python: TargetPython) -> tuple[str, ...]:
         hash_key = {
             "url": link.url_without_fragment,
             # target python participates in the hash key to handle the some cases
@@ -195,16 +198,32 @@ class WheelCache:
         hashed = hashlib.sha224(
             json.dumps(hash_key, sort_keys=True, separators=(",", ":"), ensure_ascii=True).encode("utf-8")
         ).hexdigest()
-        parts = (hashed[:2], hashed[2:4], hashed[4:6], hashed[6:])
+        return (hashed[:2], hashed[2:4], hashed[4:6], hashed[6:])
+
+    def get_path_for_link(self, link: Link, target_python: TargetPython) -> Path:
+        parts = self._get_path_parts(link, target_python)
         return self.directory.joinpath(*parts)
 
+    def get_ephemeral_path_for_link(self, link: Link, target_python: TargetPython) -> Path:
+        parts = self._get_path_parts(link, target_python)
+        return self.ephemeral_directory.joinpath(*parts)
+
     def get(self, link: Link, project_name: str | None, target_python: TargetPython) -> Path | None:
         if not project_name:
             return None
         canonical_name = canonicalize_name(project_name)
         tags_priorities = {tag: i for i, tag in enumerate(target_python.supported_tags())}
+
+        candidate = self._get_from_path(self.get_path_for_link(link, target_python), canonical_name, tags_priorities)
+        if candidate is not None:
+            return candidate
+        return self._get_from_path(
+            self.get_ephemeral_path_for_link(link, target_python), canonical_name, tags_priorities
+        )
+
+    def _get_from_path(self, path: Path, canonical_name: str, tags_priorities: dict[Tag, int]) -> Path | None:
         candidates: list[tuple[int, Path]] = []
-        for candidate in self._get_candidates(link, target_python):
+        for candidate in self._get_candidates(path):
             try:
                 name, *_, tags = parse_wheel_filename(candidate.name)
             except ValueError:
@@ -227,7 +246,7 @@ class WheelCache:
         return min(candidates, key=lambda x: x[0])[1]
 
 
-class SafeFileCache(BaseCache):
+class SafeFileCache(SeparateBodyBaseCache):
     """
     A file based cache which is safe to use even when the target directory may
     not be accessible or writable.
@@ -250,6 +269,14 @@ class SafeFileCache(BaseCache):
         with contextlib.suppress(OSError):
             with open(path, "rb") as f:
                 return f.read()
+
+        return None
+
+    def get_body(self, key: str) -> BinaryIO | None:
+        path = self._get_cache_path(key)
+        with contextlib.suppress(OSError):
+            return cast(BinaryIO, open(f"{path}.body", "rb"))
+
         return None
 
     def set(self, key: str, value: bytes, expires: int | None = None) -> None:
@@ -258,7 +285,21 @@ class SafeFileCache(BaseCache):
             with atomic_open_for_write(path, mode="wb") as f:
                 cast(BinaryIO, f).write(value)
 
+    def set_body(self, key: str, body: bytes) -> None:
+        if body is None:
+            return
+
+        path = self._get_cache_path(key)
+        with contextlib.suppress(OSError):
+            with atomic_open_for_write(f"{path}.body", mode="wb") as f:
+                cast(BinaryIO, f).write(body)
+
     def delete(self, key: str) -> None:
         path = self._get_cache_path(key)
         with contextlib.suppress(OSError):
             os.remove(path)
+
+
+@lru_cache(maxsize=128)
+def get_wheel_cache(directory: Path) -> WheelCache:
+    return WheelCache(directory)
