@@ -4,10 +4,7 @@ import dataclasses
 import posixpath
 import sys
 from functools import wraps
-from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, TypeVar, cast
-
-from unearth import Link
-from unearth.utils import LazySequence
+from typing import TYPE_CHECKING, TypeVar, cast
 
 from pdm import termui
 from pdm.exceptions import CandidateInfoNotFound, CandidateNotFound
@@ -28,9 +25,15 @@ from pdm.utils import (
 )
 
 if TYPE_CHECKING:
+    from typing import Any, Callable, Iterable, Mapping
+
+    from unearth import Link
+
     from pdm._types import CandidateInfo, RepositoryConfig, SearchResult
     from pdm.environments import BaseEnvironment
 
+    CandidateKey = tuple[str, str | None, str | None, bool]
+
 ALLOW_ALL_PYTHON = PySpecSet()
 T = TypeVar("T", bound="BaseRepository")
 
@@ -123,6 +126,8 @@ class BaseRepository:
         """Make a candidate for this package.
         In this case the finder will look for a candidate from the package sources
         """
+        from unearth import Link
+
         project = self.environment.project
         assert project.name
         link = Link.from_path(project.root)
@@ -141,6 +146,7 @@ class BaseRepository:
         """
         # `allow_prereleases` is None means leave it to specifier to decide whether to
         # include prereleases
+        from unearth.utils import LazySequence
 
         if self.is_this_package(requirement):
             return [self.make_this_candidate(requirement)]
@@ -215,6 +221,28 @@ class BaseRepository:
         summary = prepared.metadata.metadata["Summary"]
         return deps, requires_python, summary
 
+    def _get_dependency_from_local_package(self, candidate: Candidate) -> CandidateInfo:
+        """Adds the local package as a candidate only if the candidate
+        name is the same as the local package."""
+        project = self.environment.project
+        if not project.name or candidate.name != project.name:
+            raise CandidateInfoNotFound(candidate) from None
+
+        reqs = project.pyproject.metadata.get("dependencies", [])
+        extra_dependencies = project.pyproject.settings.get("dev-dependencies", {}).copy()
+        extra_dependencies.update(project.pyproject.metadata.get("optional-dependencies", {}))
+        if candidate.req.extras is not None:
+            reqs = sum(
+                (extra_dependencies.get(g, []) for g in candidate.req.extras),
+                [],
+            )
+
+        return (
+            reqs,
+            str(self.environment.python_requires),
+            project.pyproject.metadata.get("description", "UNKNOWN"),
+        )
+
     def get_hashes(self, candidate: Candidate) -> dict[Link, str] | None:
         """Get hashes of all possible installable candidates
         of a given package version.
@@ -229,7 +257,7 @@ class BaseRepository:
             return candidate.hashes
         req = candidate.req.as_pinned_version(candidate.version)
         comes_from = candidate.link.comes_from if candidate.link else None
-        result: dict[str, str] = {}
+        result: dict[Link, str] = {}
         logged = False
         respect_source_order = self.environment.project.pyproject.settings.get("resolution", {}).get(
             "respect-source-order", False
@@ -240,7 +268,7 @@ class BaseRepository:
             sources = self.sources
         with self.environment.get_finder(sources, self.ignore_compatibility) as finder:
             if req.is_file_or_url:
-                this_link = cast(Link, candidate.prepare(self.environment).link)
+                this_link = cast("Link", candidate.prepare(self.environment).link)
                 links: list[Link] = [this_link]
             else:  # the req must be a named requirement
                 links = [package.link for package in finder.find_matches(req.as_line())]
@@ -311,11 +339,14 @@ class PyPIRepository(BaseRepository):
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
         yield self._get_dependencies_from_cache
+        yield self._get_dependency_from_local_package
         if self.environment.project.config["pypi.json_api"]:
             yield self._get_dependencies_from_json
         yield self._get_dependencies_from_metadata
 
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
+        from unearth.utils import LazySequence
+
         sources = self.get_filtered_sources(requirement)
         with self.environment.get_finder(sources, self.ignore_compatibility) as finder:
             cans = LazySequence(
@@ -363,9 +394,9 @@ class LockedRepository(BaseRepository):
         environment: BaseEnvironment,
     ) -> None:
         super().__init__(sources, environment, ignore_compatibility=False)
-        self.packages: dict[tuple, Candidate] = {}
+        self.packages: dict[CandidateKey, Candidate] = {}
         self.file_hashes: dict[tuple[str, str], dict[Link, str]] = {}
-        self.candidate_info: dict[tuple, CandidateInfo] = {}
+        self.candidate_info: dict[CandidateKey, CandidateInfo] = {}
         self._read_lockfile(lockfile)
 
     @property
@@ -373,6 +404,8 @@ class LockedRepository(BaseRepository):
         return {can.req.identify(): can for can in self.packages.values()}
 
     def _read_lockfile(self, lockfile: Mapping[str, Any]) -> None:
+        from unearth import Link
+
         root = self.environment.project.root
         with cd(root):
             for package in lockfile.get("package", []):
@@ -399,7 +432,7 @@ class LockedRepository(BaseRepository):
                 Link(item["url"]): item["hash"] for item in hashes if "url" in item
             }
 
-    def _identify_candidate(self, candidate: Candidate) -> tuple:
+    def _identify_candidate(self, candidate: Candidate) -> CandidateKey:
         url = getattr(candidate.req, "url", None)
         if url is not None:
             url = url_without_fragments(url)
@@ -417,43 +450,28 @@ class LockedRepository(BaseRepository):
     def _get_dependencies_from_lockfile(self, candidate: Candidate) -> CandidateInfo:
         return self.candidate_info[self._identify_candidate(candidate)]
 
-    def _get_dependency_from_local_package(self, candidate: Candidate) -> CandidateInfo:
-        """Adds the local package as a candidate only if the candidate
-        name is the same as the local package."""
-        if candidate.name != self.environment.project.name:
-            raise CandidateInfoNotFound(candidate) from None
-
-        reqs = self.environment.project.pyproject.metadata.get("dependencies", [])
-        optional_dependencies = self.environment.project.pyproject.metadata.get("optional-dependencies", {})
-        if candidate.req.extras is not None:
-            reqs = sum(
-                (optional_dependencies.get(g, []) for g in candidate.req.extras),
-                [],
-            )
-
-        return (
-            reqs,
-            str(self.environment.python_requires),
-            self.environment.project.pyproject.metadata.get("description", "UNKNOWN"),
-        )
-
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
         return (
             self._get_dependency_from_local_package,
             self._get_dependencies_from_lockfile,
         )
 
-    def _matching_keys(self, requirement: Requirement) -> Iterable[tuple]:
+    def _matching_keys(self, requirement: Requirement) -> Iterable[CandidateKey]:
+        from pdm.models.requirements import FileRequirement
+
         for key in self.candidate_info:
+            can_req = self.packages[key].req
             if requirement.name:
                 if key[0] != requirement.identify():
                     continue
-            elif key[2] is not None:
-                if key[2] != url_without_fragments(getattr(requirement, "url", "")):
-                    continue
             else:
-                can_req = self.packages[key].req
-                if can_req.path != getattr(requirement, "path", None):  # type: ignore[attr-defined]
+                assert isinstance(requirement, FileRequirement)
+                if not isinstance(can_req, FileRequirement):
+                    continue
+                if requirement.path and can_req.path:
+                    if requirement.path != can_req.path:
+                        continue
+                elif key[2] is not None and key[2] != url_without_fragments(requirement.url):
                     continue
 
             yield key
