@@ -18,7 +18,6 @@ from packaging.requirements import InvalidRequirement
 from packaging.requirements import Requirement as PackageRequirement
 from packaging.specifiers import SpecifierSet
 from packaging.utils import parse_sdist_filename, parse_wheel_filename
-from unearth import Link
 
 from pdm.compat import Distribution
 from pdm.exceptions import ExtrasWarning, RequirementError
@@ -38,6 +37,8 @@ from pdm.utils import (
 )
 
 if TYPE_CHECKING:
+    from unearth import Link
+
     from pdm._types import RequirementDict
 
 
@@ -214,6 +215,8 @@ class Requirement:
 
     @classmethod
     def from_pkg_requirement(cls, req: PackageRequirement) -> Requirement:
+        from unearth import Link
+
         kwargs = {
             "name": req.name,
             "extras": req.extras,
@@ -229,7 +232,7 @@ class Requirement:
 
     def _format_marker(self) -> str:
         if self.marker:
-            return f"; {str(self.marker)}"
+            return f"; {self.marker!s}"
         return ""
 
 
@@ -255,6 +258,35 @@ class FileRequirement(Requirement):
     def _hash_key(self) -> tuple:
         return (*super()._hash_key(), self.get_full_url(), self.editable)
 
+    def guess_name(self) -> str | None:
+        filename = os.path.basename(urlparse.unquote(url_without_fragments(self.url))).rsplit("@", 1)[0]
+        if self.is_vcs:
+            if self.vcs == "git":  # type: ignore[attr-defined]
+                name = filename
+                if name.endswith(".git"):
+                    name = name[:-4]
+                return name
+            elif self.vcs == "hg":  # type: ignore[attr-defined]
+                return filename
+            else:  # svn and bzr
+                name, in_branch, _ = filename.rpartition("/branches/")
+                if not in_branch and name.endswith("/trunk"):
+                    return name[:-6]
+                return name
+        elif filename.endswith(".whl"):
+            return parse_wheel_filename(filename)[0]
+        else:
+            try:
+                return parse_sdist_filename(filename)[0]
+            except ValueError:
+                match = _egg_info_re.match(filename)
+                # Filename is like `<name>-<version>.tar.gz`, where name will be
+                # extracted and version will be left to be determined from
+                # the metadata.
+                if match:
+                    return match.group(1)
+        return None
+
     @classmethod
     def create(cls: type[T], **kwargs: Any) -> T:
         if kwargs.get("path"):
@@ -311,6 +343,8 @@ class FileRequirement(Requirement):
         return self.is_local and cast(Path, self.path).is_dir()
 
     def as_file_link(self) -> Link:
+        from unearth import Link
+
         url = self.get_full_url()
         # only subdirectory is useful in a file link
         if self.subdirectory:
@@ -349,19 +383,7 @@ class FileRequirement(Requirement):
             if not self.extras:
                 self.extras = extras
         if not self.name and not self.is_vcs:
-            filename = os.path.basename(urlparse.unquote(url_without_fragments(self.url)))
-            if filename.endswith(".whl"):
-                self.name, *_ = parse_wheel_filename(filename)
-            else:
-                try:
-                    self.name, *_ = parse_sdist_filename(filename)
-                except ValueError:
-                    match = _egg_info_re.match(filename)
-                    # Filename is like `<name>-<version>.tar.gz`, where name will be
-                    # extracted and version will be left to be determined from
-                    # the metadata.
-                    if match:
-                        self.name = match.group(1)
+            self.name = self.guess_name()
 
     def _check_installable(self) -> None:
         assert self.path
@@ -474,7 +496,7 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
             if m is None:
                 raise RequirementError(str(e)) from None
             args = m.groupdict()
-            if not args["url"] and args["path"] and not os.path.exists(args["path"]):
+            if not line.startswith(".") and not args["url"] and args["path"] and not os.path.exists(args["path"]):
                 raise RequirementError(str(e)) from None
             r = FileRequirement.create(**args)
         else:
