@@ -11,77 +11,32 @@ import rich.theme
 import tomlkit
 
 from pdm import termui
-from pdm._types import Source
+from pdm._types import RepositoryConfig
+from pdm.compat import cached_property
 from pdm.exceptions import NoConfigError, PdmUsageError
 
-ui = termui.UI()
-
 REPOSITORY = "repository"
-
-
-@dataclasses.dataclass
-class RepositoryConfig:
-    url: str
-    username: str | None = None
-    password: str | None = None
-    ca_certs: str | None = None
-
-    config_prefix: str | None = None
-
-    def __rich__(self) -> str:
-        config_prefix = f"{self.config_prefix}." if self.config_prefix is not None else ""
-        lines = [f"[primary]{config_prefix}url[/] = {self.url}"]
-        if self.username:
-            lines.append(f"[primary]{config_prefix}username[/] = {self.username}")
-        if self.password:
-            lines.append(f"[primary]{config_prefix}password[/] = [i]<hidden>[/]")
-        if self.ca_certs:
-            lines.append(f"[primary]{config_prefix}ca_certs[/] = {self.ca_certs}")
-        return "\n".join(lines)
-
-
-@dataclasses.dataclass
-class RegistryConfig:
-    url: str
-    username: str | None = None
-    password: str | None = None
-    verify_ssl: bool | None = None
-    type: str | None = None
-
-    config_prefix: str | None = None
-
-    def __rich__(self) -> str:
-        config_prefix = f"{self.config_prefix}." if self.config_prefix is not None else ""
-        lines = [f"[primary]{config_prefix}url[/] = {self.url}"]
-        if self.username:
-            lines.append(f"[primary]{config_prefix}username[/] = {self.username}")
-        if self.password:
-            lines.append(f"[primary]{config_prefix}password[/] = [i]<hidden>[/]")
-        if self.verify_ssl:
-            lines.append(f"[primary]{config_prefix}verify_ssl[/] = {self.verify_ssl}")
-        if self.type:
-            lines.append(f"[primary]{config_prefix}type[/] = {self.type}")
-        return "\n".join(lines)
-
-
+SOURCE = "pypi"
 DEFAULT_REPOSITORIES = {
-    "pypi": RepositoryConfig("https://upload.pypi.org/legacy/"),
-    "testpypi": RepositoryConfig("https://test.pypi.org/legacy/"),
+    "pypi": "https://upload.pypi.org/legacy/",
+    "testpypi": "https://test.pypi.org/legacy/",
 }
 
+ui = termui.UI()
+
 
 def load_config(file_path: Path) -> dict[str, Any]:
     """Load a nested TOML document into key-value pairs
 
-    E.g. ["python"]["path"] will be loaded as "python.path" key.
+    E.g. ["python"]["use_venv"] will be loaded as "python.use_venv" key.
     """
 
     def get_item(sub_data: Mapping[str, Any]) -> dict[str, Any]:
         result: dict[str, Any] = {}
         for k, v in sub_data.items():
-            if k == "pypi":
+            if k in (REPOSITORY, SOURCE):
                 result.update((f"{k}.{sub_k}", sub_v) for sub_k, sub_v in v.items())
-            elif k != REPOSITORY and isinstance(v, Mapping):
+            elif isinstance(v, Mapping):
                 result.update({f"{k}.{sub_k}": sub_v for sub_k, sub_v in get_item(v).items()})
             else:
                 result.update({k: v})
@@ -200,13 +155,9 @@ class Config(MutableMapping[str, str]):
             "`symlink` or `pth` to create links to the cached installation",
             "symlink",
         ),
-        "python.path": ConfigItem("The Python interpreter path", env_var="PDM_PYTHON"),
         "python.use_pyenv": ConfigItem("Use the pyenv interpreter", True, coerce=ensure_boolean),
         "python.use_venv": ConfigItem(
-            "Install packages into the activated venv site packages instead of PEP 582",
-            True,
-            env_var="PDM_USE_VENV",
-            coerce=ensure_boolean,
+            "Use virtual environments when available", True, env_var="PDM_USE_VENV", coerce=ensure_boolean
         ),
         "pypi.url": ConfigItem(
             "The URL of PyPI mirror, defaults to https://pypi.org/simple",
@@ -217,7 +168,7 @@ class Config(MutableMapping[str, str]):
         "pypi.username": ConfigItem("The username to access PyPI", env_var="PDM_PYPI_USERNAME"),
         "pypi.password": ConfigItem("The password to access PyPI", env_var="PDM_PYPI_PASSWORD"),
         "pypi.ca_certs": ConfigItem(
-            "Path to a CA certificate bundle used for verifying the identity of the PyPI server",
+            "Path to a CA certificate bundle used for verifying the identity of the PyPI server", global_only=True
         ),
         "pypi.ignore_stored_index": ConfigItem(
             "Ignore the configured indexes",
@@ -225,12 +176,8 @@ class Config(MutableMapping[str, str]):
             env_var="PDM_IGNORE_STORED_INDEX",
             coerce=ensure_boolean,
         ),
-        "pypi.client_cert": ConfigItem(
-            "Path to client certificate file, or combined cert/key file",
-        ),
-        "pypi.client_key": ConfigItem(
-            "Path to client cert keyfile, if not in pypi.client_cert",
-        ),
+        "pypi.client_cert": ConfigItem("Path to client certificate file, or combined cert/key file", global_only=True),
+        "pypi.client_key": ConfigItem("Path to client cert keyfile, if not in pypi.client_cert", global_only=True),
         "pypi.json_api": ConfigItem(
             "Consult PyPI's JSON API for package metadata",
             False,
@@ -282,6 +229,10 @@ class Config(MutableMapping[str, str]):
         defaults.update(cls.site)
         return defaults
 
+    @cached_property
+    def env_map(self) -> Mapping[str, Any]:
+        return EnvMap(self._config_map)
+
     @classmethod
     def add_config(cls, name: str, item: ConfigItem) -> None:
         """Add or modify a config item"""
@@ -292,7 +243,11 @@ class Config(MutableMapping[str, str]):
         self.config_file = config_file.resolve()
         self.deprecated = {v.replace: k for k, v in self._config_map.items() if v.replace}
         self._file_data = load_config(self.config_file)
-        self._data = collections.ChainMap(self._file_data, self.get_defaults() if is_global else {})
+        self._data = collections.ChainMap(
+            cast(MutableMapping[str, Any], self.env_map) if is_global else {},
+            self._file_data,
+            self.get_defaults() if is_global else {},
+        )
 
     def load_theme(self) -> rich.theme.Theme:
         if not self.is_global:  # pragma: no cover
@@ -303,10 +258,10 @@ class Config(MutableMapping[str, str]):
     def self_data(self) -> dict[str, Any]:
         return dict(self._file_data)
 
-    def iter_sources(self) -> Iterator[tuple[str, Source]]:
+    def iter_sources(self) -> Iterator[RepositoryConfig]:
         for name, data in self._data.items():
-            if name.startswith("pypi.") and name not in self._config_map:
-                yield name[5:], cast(Source, dict(data, name=name))
+            if name.startswith(f"{SOURCE}.") and name not in self._config_map:
+                yield RepositoryConfig(**data, name=name[len(SOURCE) + 1 :], config_prefix=SOURCE)
 
     def _save_config(self) -> None:
         """Save the changed to config file."""
@@ -326,59 +281,47 @@ class Config(MutableMapping[str, str]):
 
     def __getitem__(self, key: str) -> Any:
         parts = key.split(".")
-        if parts[0] == REPOSITORY:
+        if parts[0] in (REPOSITORY, SOURCE) and key not in self._config_map:
             if len(parts) < 2:
-                raise PdmUsageError("Must specify a repository name")
-            repo = self.get_repository_config(parts[1])
+                raise PdmUsageError(f"Must specify a {parts[0]} name")
+            repo = self.get_repository_config(parts[1], parts[0])
             if repo is None:
-                raise KeyError(f"No repository named {parts[1]}")
+                raise KeyError(f"No {parts[0]} named {parts[1]}")
 
             value = getattr(repo, parts[2]) if len(parts) >= 3 else repo
-            if len(parts) >= 3 and parts[2] == "password" and value:
-                return "<hidden>"
             return value
-        elif parts[0] == "pypi" and key not in self._config_map:
-            index_key = ".".join(parts[:2])
-            if index_key not in self._data:
-                raise KeyError(f"No PyPI index named {parts[1]}")
-            source = self._data[index_key]
-            if len(parts) >= 3 and parts[2] == "password":
-                return "<hidden>"
-            return source[parts[2]] if len(parts) >= 3 else RegistryConfig(**self._data[index_key])
-        elif key == "pypi.password":
-            return "<hidden>"
 
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
         config = self._config_map[config_key]
-        env_var = config.env_var
-        if env_var is not None and env_var in os.environ:
-            result = os.environ[env_var]
+
+        if config_key in self._data:
+            result = self._data[config_key]
+        elif config.replace:
+            result = self._data[config.replace]
         else:
-            if config_key in self._data:
-                result = self._data[config_key]
-            elif config.replace:
-                result = self._data[config.replace]
-            else:
-                raise NoConfigError(key) from None
+            raise NoConfigError(key) from None
         return config.coerce(result)
 
     def __setitem__(self, key: str, value: Any) -> None:
+        from pdm.models.auth import keyring
+
         parts = key.split(".")
-        if parts[0] == REPOSITORY:
+        if parts[0] in (REPOSITORY, SOURCE) and key not in self._config_map:
             if len(parts) < 3:
-                raise PdmUsageError("Set repository config with [success]repository.{name}.{attr}")
-            self._file_data.setdefault(parts[0], {}).setdefault(parts[1], {}).setdefault(parts[2], value)
-            self._save_config()
-            return
-        if parts[0] == "pypi" and key not in self._config_map:
-            if len(parts) < 3:
-                raise PdmUsageError("Set index config with [success]pypi.{name}.{attr}")
+                raise PdmUsageError(f"Set {parts[0]} config with [success]{parts[0]}.{{name}}.{{attr}}")
             index_key = ".".join(parts[:2])
+            username = self._data.get(index_key, {}).get("username")
+            service = f'pdm-{index_key.replace(".", "-")}'
+            if parts[2] == "password" and self.is_global and keyring.save_auth_info(service, username, value):
+                return
+            if parts[2] == "verify_ssl":
+                value = ensure_boolean(value)
             self._file_data.setdefault(index_key, {})[parts[2]] = value
             self._save_config()
             return
+
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
@@ -387,10 +330,9 @@ class Config(MutableMapping[str, str]):
             raise ValueError(f"Config item '{key}' is not allowed to set in project config.")
 
         value = config.coerce(value)
-        env_var = config.env_var
-        if env_var is not None and env_var in os.environ:
+        if key in self.env_map:
             ui.echo(
-                "WARNING: the config is shadowed by env var '{}', the value set won't take effect.".format(env_var),
+                f"WARNING: the config is shadowed by env var '{config.env_var}', the value set won't take effect.",
                 style="warning",
             )
         self._file_data[config_key] = value
@@ -405,25 +347,15 @@ class Config(MutableMapping[str, str]):
         keys: set[str] = set()
         for key in self._data:
             if key in self.deprecated:
-                keys.add(self.deprecated[key])
-            elif key != REPOSITORY:
-                keys.add(key)
+                key = self.deprecated[key]
+            keys.add(key)
         return iter(keys)
 
     def __delitem__(self, key: str) -> None:
         parts = key.split(".")
-        if parts[0] == REPOSITORY:
+        if parts[0] in (REPOSITORY, SOURCE) and key not in self._config_map:
             if len(parts) < 2:
-                raise PdmUsageError("Should specify the name of repository")
-            if len(parts) >= 3:
-                del self._file_data.get(REPOSITORY, {}).get(parts[1], {})[parts[2]]
-            else:
-                del self._file_data.get(REPOSITORY, {})[parts[1]]
-            self._save_config()
-            return
-        if parts[0] == "pypi" and key not in self._config_map:
-            if len(parts) < 2:
-                raise PdmUsageError("Should specify the name of index")
+                raise PdmUsageError(f"Should specify the name of {parts[0]}")
             if len(parts) >= 3:
                 index_key, attr = key.rsplit(".", 1)
                 del self._file_data.get(index_key, {})[attr]
@@ -431,6 +363,7 @@ class Config(MutableMapping[str, str]):
                 del self._file_data[key]
             self._save_config()
             return
+
         config_key = self.deprecated.get(key, key)
         config = self._config_map[config_key]
         self._file_data.pop(config_key, None)
@@ -445,24 +378,66 @@ class Config(MutableMapping[str, str]):
             )
         self._save_config()
 
-    def get_repository_config(self, name_or_url: str) -> RepositoryConfig | None:
-        """Get a repository by name or url."""
-        if not self.is_global:  # pragma: no cover
-            raise NoConfigError("repository")
-        repositories: Mapping[str, Source] = self._data.get(REPOSITORY, {})
-        repo: RepositoryConfig | None = None
+    def get_repository_config(self, name_or_url: str, prefix: str) -> RepositoryConfig | None:
+        """Get a repository or source by name or url."""
+        if not self.is_global and prefix == REPOSITORY:  # pragma: no cover
+            raise NoConfigError(prefix)
+        repositories: dict[str, RepositoryConfig] = {}
+        for k, v in self._data.items():
+            if not k.startswith(f"{prefix}.") or k in self._config_map:
+                continue
+            key = k[len(prefix) + 1 :]
+            repositories[key] = RepositoryConfig(**v, name=key, config_prefix=prefix)
+        config: RepositoryConfig | None = None
         if "://" in name_or_url:
-            config: Source = next((v for v in repositories.values() if v.get("url") == name_or_url), {})
-            repo = next(
-                (r for r in DEFAULT_REPOSITORIES.values() if r.url == name_or_url),
-                RepositoryConfig(name_or_url),
+            config = next(
+                (v for v in repositories.values() if v.url == name_or_url),
+                RepositoryConfig(url=name_or_url, name="__unknown__", config_prefix=prefix),
             )
         else:
-            config = repositories.get(name_or_url, {})
-            if name_or_url in DEFAULT_REPOSITORIES:
-                repo = DEFAULT_REPOSITORIES[name_or_url]
-        if repo:
-            return dataclasses.replace(repo, **config)
-        if not config:
-            return None
-        return RepositoryConfig(**config)  # type: ignore[misc]
+            config = repositories.get(name_or_url)
+
+        if prefix == SOURCE:
+            return config
+
+        if name_or_url in DEFAULT_REPOSITORIES:
+            if config is None:
+                return RepositoryConfig(url=DEFAULT_REPOSITORIES[name_or_url], name=name_or_url, config_prefix=prefix)
+            config.passive_update(url=DEFAULT_REPOSITORIES[name_or_url])
+        if name_or_url in DEFAULT_REPOSITORIES.values():
+            name = next(k for k, v in DEFAULT_REPOSITORIES.items() if v == name_or_url)
+            if config is None:
+                return RepositoryConfig(
+                    name=name,
+                    config_prefix=prefix,
+                    url=name_or_url,
+                )
+            config.passive_update(url=name_or_url)
+            if config.name == "__unknown__":
+                config.name = name
+        return config
+
+
+class EnvMap(Mapping[str, Any]):
+    def __init__(self, config_items: Mapping[str, ConfigItem]) -> None:
+        self._config_map = config_items
+
+    def __repr__(self) -> str:
+        return repr(dict(self))
+
+    def __getitem__(self, k: str) -> Any:
+        try:
+            item = self._config_map[k]
+            if item.env_var:
+                return item.coerce(os.environ[item.env_var])
+        except KeyError:
+            pass
+        raise KeyError(k)
+
+    def __iter__(self) -> Iterator[str]:
+        for key, item in self._config_map.items():
+            if item.env_var and item.env_var in os.environ:
+                yield key
+
+    def __len__(self) -> int:
+        return sum(1 for _ in self)
