@@ -1,30 +1,26 @@
 from __future__ import annotations
 
+import contextlib
 import hashlib
 import os
 import re
 import shutil
 import sys
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Iterable, cast
+from typing import TYPE_CHECKING, Any, Iterable, Mapping, cast
 
-import platformdirs
 import tomlkit
-from findpython import Finder
 from tomlkit.items import Array
-from unearth import Link
 
 from pdm import termui
+from pdm._types import RepositoryConfig
 from pdm.compat import cached_property
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.models.backends import BuildBackend, get_backend_by_spec
-from pdm.models.caches import CandidateInfoCache, HashCache, WheelCache
-from pdm.models.candidates import Candidate, make_candidate
-from pdm.models.environment import Environment, GlobalEnvironment
 from pdm.models.python import PythonInfo
 from pdm.models.repositories import BaseRepository, LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
-from pdm.models.specifiers import PySpecSet, get_specifier
+from pdm.models.specifiers import PySpecSet
 from pdm.project.config import Config
 from pdm.project.lockfile import Lockfile
 from pdm.project.project_file import PyProject
@@ -34,16 +30,19 @@ from pdm.utils import (
     expand_env_vars_in_auth,
     find_project_root,
     find_python_in_path,
-    get_venv_like_prefix,
     normalize_name,
     path_to_url,
 )
 
 if TYPE_CHECKING:
+    from findpython import Finder
     from resolvelib.reporters import BaseReporter
 
-    from pdm._types import Source, Spinner
+    from pdm._types import Spinner
     from pdm.core import Core
+    from pdm.environments import BaseEnvironment
+    from pdm.models.caches import CandidateInfoCache, HashCache, WheelCache
+    from pdm.models.candidates import Candidate
     from pdm.resolver.providers import BaseProvider
 
 
@@ -71,8 +70,10 @@ class Project:
         is_global: bool = False,
         global_config: str | Path | None = None,
     ) -> None:
+        import platformdirs
+
         self._lockfile: Lockfile | None = None
-        self._environment: Environment | None = None
+        self._environment: BaseEnvironment | None = None
         self._python: PythonInfo | None = None
         self.core = core
 
@@ -117,12 +118,12 @@ class Project:
     def set_lockfile(self, path: str | Path) -> None:
         self._lockfile = Lockfile(path, ui=self.core.ui)
 
-    @property
-    def config(self) -> dict[str, Any]:
-        """A read-only dict configuration, any modifications won't land in the file."""
-        result = dict(self.global_config)
-        result.update(self.project_config)
-        return result
+    @cached_property
+    def config(self) -> Mapping[str, Any]:
+        """A read-only dict configuration"""
+        import collections
+
+        return collections.ChainMap(self.project_config, self.global_config)
 
     @property
     def scripts(self) -> dict[str, str | dict[str, str]]:
@@ -131,7 +132,12 @@ class Project:
     @cached_property
     def project_config(self) -> Config:
         """Read-and-writable configuration dict for project settings"""
-        return Config(self.root / ".pdm.toml")
+        config = Config(self.root / "pdm.toml")
+        # TODO: for backward compatibility, remove this in the future
+        if self.root.joinpath(".pdm.toml").exists():
+            legacy_config = Config(self.root / ".pdm.toml").self_data
+            config.update((k, v) for k, v in legacy_config.items() if k != "python.path")
+        return config
 
     @property
     def name(self) -> str | None:
@@ -151,47 +157,80 @@ class Project:
     @python.setter
     def python(self, value: PythonInfo) -> None:
         self._python = value
-        self.project_config["python.path"] = value.path
+        self._saved_python = value.path.as_posix()
+
+    @property
+    def _saved_python(self) -> str | None:
+        if os.getenv("PDM_PYTHON"):
+            return os.getenv("PDM_PYTHON")
+        with contextlib.suppress(FileNotFoundError):
+            return self.root.joinpath(".pdm-python").read_text("utf-8").strip()
+        with contextlib.suppress(FileNotFoundError):
+            # TODO: remove this in the future
+            with self.root.joinpath(".pdm.toml").open("rb") as fp:
+                data = tomlkit.load(fp)
+                if data.get("python", {}).get("path"):
+                    return data["python"]["path"]
+        return None
+
+    @_saved_python.setter
+    def _saved_python(self, value: str | None) -> None:
+        self.root.mkdir(parents=True, exist_ok=True)
+        python_file = self.root.joinpath(".pdm-python")
+        if value is None:
+            with contextlib.suppress(FileNotFoundError):
+                python_file.unlink()
+            return
+        python_file.write_text(value, "utf-8")
 
     def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
-        from pdm.cli.commands.venv.utils import get_venv_python, iter_venvs
+        from pdm.cli.commands.venv.utils import iter_venvs
+        from pdm.models.venv import get_venv_python
 
         def match_version(python: PythonInfo) -> bool:
             return python.valid and self.python_requires.contains(python.version, True)
 
         def note(message: str) -> None:
             if not self.is_global:
-                self.core.ui.echo(message, style="warning", err=True)
+                self.core.ui.echo(message, style="info", err=True)
 
         config = self.config
-        if config.get("python.path") and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
-            saved_path = config["python.path"]
+        saved_path = self._saved_python
+        if saved_path and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
             python = PythonInfo.from_path(saved_path)
             if match_version(python):
                 return python
-            self.project_config.pop("python.path", None)
+            else:
+                note(
+                    "The saved Python interpreter doesn't match the project's requirement. "
+                    "Trying to find another one."
+                )
+            self._saved_python = None  # Clear the saved path if it doesn't match
 
-        if config.get("python.use_venv") and not self.is_global:
+        if config.get("python.use_venv") and not self.is_global and not os.getenv("PDM_IGNORE_ACTIVE_VENV"):
             # Resolve virtual environments from env-vars
             venv_in_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
             if venv_in_env:
                 python = PythonInfo.from_path(get_venv_python(Path(venv_in_env)))
                 if match_version(python):
-                    note(f"Inside an active virtualenv [success]{venv_in_env}[/], reusing it.")
+                    note(
+                        f"Inside an active virtualenv [success]{venv_in_env}[/], reusing it.\n"
+                        "Set env var [success]PDM_IGNORE_ACTIVE_VENV[/] to ignore it."
+                    )
                     return python
             # otherwise, get a venv associated with the project
             for _, venv in iter_venvs(self):
-                python = PythonInfo.from_path(get_venv_python(venv))
+                python = PythonInfo.from_path(venv.interpreter)
                 if match_version(python):
-                    note(f"Virtualenv [success]{venv}[/] is reused.")
+                    note(f"Virtualenv [success]{venv.root}[/] is reused.")
                     self.python = python
                     return python
 
             if not self.root.joinpath("__pypackages__").exists():
                 note("python.use_venv is on, creating a virtualenv for this project...")
-                venv = self._create_virtualenv()
-                self.python = PythonInfo.from_path(get_venv_python(venv))
+                venv_path = self._create_virtualenv()
+                self.python = PythonInfo.from_path(get_venv_python(venv_path))
                 return self.python
 
         for py_version in self.find_interpreters():
@@ -203,20 +242,22 @@ class Project:
 
         raise NoPythonVersion(f"No Python that satisfies {self.python_requires} is found on the system.")
 
-    def get_environment(self) -> Environment:
+    def get_environment(self) -> BaseEnvironment:
+        from pdm.environments import PythonEnvironment, PythonLocalEnvironment
+
         """Get the environment selected by this project"""
 
         if self.is_global:
-            env = GlobalEnvironment(self)
+            env = PythonEnvironment(self)
             # Rewrite global project's python requires to be
             # compatible with the exact version
             env.python_requires = PySpecSet(f"=={self.python.version}")
             return env
 
         return (
-            GlobalEnvironment(self)
-            if self.config["python.use_venv"] and get_venv_like_prefix(self.python.executable) is not None
-            else Environment(self)
+            PythonEnvironment(self)
+            if self.config["python.use_venv"] and self.python.get_venv() is not None
+            else PythonLocalEnvironment(self)
         )
 
     def _create_virtualenv(self) -> Path:
@@ -234,13 +275,13 @@ class Project:
         return path
 
     @property
-    def environment(self) -> Environment:
+    def environment(self) -> BaseEnvironment:
         if not self._environment:
             self._environment = self.get_environment()
         return self._environment
 
     @environment.setter
-    def environment(self, value: Environment) -> None:
+    def environment(self, value: BaseEnvironment) -> None:
         self._environment = value
 
     @property
@@ -289,27 +330,6 @@ class Project:
                 result[req.identify()] = req
         return result
 
-    @property
-    def dependencies(self) -> dict[str, Requirement]:
-        return self.get_dependencies()
-
-    @property
-    def dev_dependencies(self) -> dict[str, Requirement]:
-        """All development dependencies"""
-        dev_group = self.pyproject.settings.get("dev-dependencies", {})
-        if not dev_group:
-            return {}
-        result = {}
-        with cd(self.root):
-            for _, deps in dev_group.items():
-                for line in deps:
-                    if line.startswith("-e "):
-                        req = parse_requirement(line[3:].strip(), True)
-                    else:
-                        req = parse_requirement(line)
-                    result[req.identify()] = req
-        return result
-
     def iter_groups(self) -> Iterable[str]:
         groups = {"default"}
         if self.pyproject.metadata.get("optional-dependencies"):
@@ -327,33 +347,42 @@ class Project:
         return self.pyproject.settings.get("allow_prereleases")
 
     @property
-    def default_source(self) -> Source:
+    def default_source(self) -> RepositoryConfig:
         """Get the default source from the pypi setting"""
-        return cast(
-            "Source",
-            {
-                "url": self.config["pypi.url"],
-                "verify_ssl": self.config["pypi.verify_ssl"],
-                "name": "pypi",
-                "username": self.config.get("pypi.username"),
-                "password": self.config.get("pypi.password"),
-            },
+        return RepositoryConfig(
+            config_prefix="pypi",
+            name="pypi",
+            url=self.config["pypi.url"],
+            verify_ssl=self.config["pypi.verify_ssl"],
+            username=self.config.get("pypi.username"),
+            password=self.config.get("pypi.password"),
         )
 
     @property
-    def sources(self) -> list[Source]:
-        sources = list(self.pyproject.settings.get("source", []))
+    def sources(self) -> list[RepositoryConfig]:
+        result: dict[str, RepositoryConfig] = {}
+        for source in self.pyproject.settings.get("source", []):
+            result[source["name"]] = RepositoryConfig(**source, config_prefix="pypi")
+
+        def merge_sources(other_sources: Iterable[RepositoryConfig]) -> None:
+            for source in other_sources:
+                name = source.name
+                if name in result:
+                    result[name].passive_update(source)
+                else:
+                    result[name] = source
+
         if not self.config.get("pypi.ignore_stored_index", False):
-            if all(source.get("name") != "pypi" for source in sources):
-                sources.insert(0, self.default_source)
-            stored_sources = dict(self.project_config.iter_sources())
-            stored_sources.update((k, v) for k, v in self.global_config.iter_sources() if k not in stored_sources)
-            # The order is kept as project sources -> global sources
-            sources.extend(stored_sources.values())
-        expanded_sources = [
-            cast("Source", {**source, "url": expand_env_vars_in_auth(source["url"])}) for source in sources
-        ]
-        return expanded_sources
+            if "pypi" not in result:  # put pypi source at the beginning
+                result = {"pypi": self.default_source, **result}
+            else:
+                result["pypi"].passive_update(self.default_source)
+            merge_sources(self.project_config.iter_sources())
+            merge_sources(self.global_config.iter_sources())
+        for source in result.values():
+            assert source.url, "Source URL must not be empty"
+            source.url = expand_env_vars_in_auth(source.url)
+        return list(result.values())
 
     def get_repository(
         self, cls: type[BaseRepository] | None = None, ignore_compatibility: bool = True
@@ -444,22 +473,42 @@ class Project:
 
         return SpinnerReporter(spinner, requirements)
 
-    def get_lock_metadata(self) -> dict[str, Any]:
+    def get_lock_metadata(
+        self, groups: Iterable[str] | None = None, cross_platform: bool | None = None
+    ) -> dict[str, Any]:
         content_hash = tomlkit.string("sha256:" + self.pyproject.content_hash("sha256"))
         content_hash.trivia.trail = "\n\n"
+        if groups is None:
+            groups = self.iter_groups()
+        if cross_platform is None:
+            cross_platform = self.lockfile.cross_platform
         return {
             "lock_version": self.lockfile.spec_version,
+            "cross_platform": cross_platform,
+            "groups": sorted(groups, key=lambda x: (x != "default", x)),
             "content_hash": content_hash,
         }
 
-    def write_lockfile(self, toml_data: dict, show_message: bool = True, write: bool = True) -> None:
-        toml_data["metadata"].update(self.get_lock_metadata())
+    def write_lockfile(
+        self,
+        toml_data: dict,
+        show_message: bool = True,
+        write: bool = True,
+        groups: Iterable[str] | None = None,
+        cross_platform: bool | None = None,
+    ) -> None:
+        """Write the lock file to disk."""
+        toml_data["metadata"].update(self.get_lock_metadata(groups, cross_platform=cross_platform))
         self.lockfile.set_data(toml_data)
 
         if write:
             self.lockfile.write(show_message)
 
     def make_self_candidate(self, editable: bool = True) -> Candidate:
+        from unearth import Link
+
+        from pdm.models.candidates import make_candidate
+
         req = parse_requirement(path_to_url(self.root.as_posix()), editable)
         assert self.name
         req.name = self.name
@@ -475,20 +524,6 @@ class Project:
         content_hash = self.pyproject.content_hash(algo)
         return content_hash == hash_value
 
-    def is_lockfile_compatible(self) -> bool:
-        """Within the same major version, the higher lockfile generator can work with
-        lower lockfile but not vice versa.
-        """
-        if not self.lockfile.exists:
-            return True
-        lockfile_version = str(self.lockfile.file_version)
-        if not lockfile_version:
-            return False
-        if "." not in lockfile_version:
-            lockfile_version += ".0"
-        accepted = get_specifier(f"~={lockfile_version},>={lockfile_version}")
-        return accepted.contains(self.lockfile.spec_version)
-
     def get_pyproject_dependencies(self, group: str, dev: bool = False) -> tuple[list[str], bool]:
         """Get the dependencies array in the pyproject.toml
         Return a tuple of two elements, the first is the dependencies array,
@@ -537,7 +572,7 @@ class Project:
         self.pyproject.write(show_message)
 
     def init_global_project(self) -> None:
-        if not self.is_global or not self.pyproject.empty:
+        if not self.is_global or not self.pyproject.empty():
             return
         self.root.mkdir(parents=True, exist_ok=True)
         self.pyproject.set_data({"project": {"dependencies": ["pip", "setuptools", "wheel"]}})
@@ -561,14 +596,20 @@ class Project:
         return path
 
     def make_wheel_cache(self) -> WheelCache:
-        return WheelCache(self.cache("wheels"))
+        from pdm.models.caches import get_wheel_cache
+
+        return get_wheel_cache(self.cache("wheels"))
 
     def make_candidate_info_cache(self) -> CandidateInfoCache:
+        from pdm.models.caches import CandidateInfoCache
+
         python_hash = hashlib.sha1(str(self.environment.python_requires).encode()).hexdigest()
         file_name = f"package_meta_{python_hash}.json"
         return CandidateInfoCache(self.cache("metadata") / file_name)
 
     def make_hash_cache(self) -> HashCache:
+        from pdm.models.caches import HashCache
+
         return HashCache(directory=self.cache("hashes"))
 
     def find_interpreters(self, python_spec: str | None = None) -> Iterable[PythonInfo]:
@@ -617,11 +658,13 @@ class Project:
             yield PythonInfo.from_path(this_python)
 
     def _get_python_finder(self) -> Finder:
+        from findpython import Finder
+
         from pdm.cli.commands.venv.utils import VenvProvider
 
         finder = Finder(resolve_symlinks=True)
         if self.config["python.use_venv"]:
-            finder._providers.insert(0, VenvProvider(self))
+            finder.add_provider(VenvProvider(self), 0)
         return finder
 
     # compatibility, shouldn't be used directly
