@@ -49,11 +49,9 @@ from packaging.version import parse as parse_version
 from pytest_mock import MockerFixture
 from unearth import Link
 
-from pdm.cli.actions import do_init
 from pdm.cli.hooks import HookManager
-from pdm.compat import Protocol
 from pdm.core import Core
-from pdm.environments import BaseEnvironment, PrefixEnvironment
+from pdm.environments import BaseEnvironment, PythonEnvironment
 from pdm.exceptions import CandidateInfoNotFound
 from pdm.installers.installers import install_wheel
 from pdm.models.backends import get_backend
@@ -70,6 +68,8 @@ from pdm.project.core import Project
 from pdm.utils import find_python_in_path, normalize_name, path_to_url
 
 if TYPE_CHECKING:
+    from typing import Protocol
+
     from _pytest.fixtures import SubRequest
 
     from pdm._types import CandidateInfo, RepositoryConfig
@@ -187,6 +187,7 @@ class TestRepository(BaseRepository):
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
         return (
             self._get_dependencies_from_cache,
+            self._get_dependency_from_local_package,
             self._get_dependencies_from_fixture,
             self._get_dependencies_from_metadata,
         )
@@ -319,7 +320,7 @@ def build_env(build_env_wheels: Iterable[Path], tmp_path_factory: pytest.TempPat
     """
     d = tmp_path_factory.mktemp("pdm-test-env")
     p = Core().create_project(d)
-    env = PrefixEnvironment(p, prefix=str(d))
+    env = PythonEnvironment(p, prefix=str(d))
     for wheel in build_env_wheels:
         install_wheel(str(wheel), env)
     return d
@@ -336,10 +337,13 @@ def pypi_indexes() -> IndexesDefinition:
     return {}
 
 
+_build_session = BaseEnvironment._build_session
+
+
 @pytest.fixture
 def pdm_session(pypi_indexes: IndexesDefinition) -> Callable[[Any], PDMSession]:
     def get_pypi_session(*args: Any, **kwargs: Any) -> PDMSession:
-        session = PDMSession(*args, **kwargs)
+        session = _build_session(*args, **kwargs)
         for root, specs in pypi_indexes.items():
             index, overrides, strip = specs if isinstance(specs, tuple) else (specs, None, False)
             session.mount(root, LocalFileAdapter(index, overrides=overrides, strip_suffix=strip))
@@ -388,7 +392,7 @@ def project_no_init(
     )
     p = core.create_project(tmp_path, global_config=test_home.joinpath("config.toml").as_posix())
     p.global_config["venv.location"] = str(tmp_path / "venvs")
-    mocker.patch("pdm.environments.base.PDMSession", pdm_session)
+    mocker.patch.object(BaseEnvironment, "_build_session", pdm_session)
     mocker.patch("pdm.builders.base.EnvBuilder.get_shared_env", return_value=str(build_env))
     tmp_path.joinpath("caches").mkdir(parents=True)
     p.global_config["cache_dir"] = tmp_path.joinpath("caches").as_posix()
@@ -415,8 +419,10 @@ def project(project_no_init: Project) -> Project:
     Returns:
         The initialized project
     """
+    from pdm.cli.commands.init import Command
+
     hooks = HookManager(project_no_init, ["post_init"])
-    do_init(
+    Command.do_init(
         project_no_init,
         "test_project",
         "0.0.0",
@@ -546,30 +552,32 @@ class RunResult:
         print("# stderr:", self.stderr, sep="\n")
 
 
-class PDMCallable(Protocol):
-    """The PDM fixture callable signature"""
+if TYPE_CHECKING:
 
-    def __call__(
-        self,
-        args: str | list[str],
-        strict: bool = False,
-        input: str | None = None,
-        obj: Project | None = None,
-        env: Mapping[str, str] | None = None,
-        **kwargs: Any,
-    ) -> RunResult:
-        """
-        Args:
-            args: the command arguments as a single lexable string or a strings array
-            strict: raise an exception on failure instead of returning if enabled
-            input: an optional string to be submitted too `stdin`
-            obj: an optional existing `Project`.
-            env: override the environment variables with those
+    class PDMCallable(Protocol):
+        """The PDM fixture callable signature"""
 
-        Returns:
-            The command result
-        """
-        ...
+        def __call__(
+            self,
+            args: str | list[str],
+            strict: bool = False,
+            input: str | None = None,
+            obj: Project | None = None,
+            env: Mapping[str, str] | None = None,
+            **kwargs: Any,
+        ) -> RunResult:
+            """
+            Args:
+                args: the command arguments as a single lexable string or a strings array
+                strict: raise an exception on failure instead of returning if enabled
+                input: an optional string to be submitted too `stdin`
+                obj: an optional existing `Project`.
+                env: override the environment variables with those
+
+            Returns:
+                The command result
+            """
+            ...
 
 
 @pytest.fixture
