@@ -17,13 +17,20 @@ import tempfile
 import urllib.parse as parse
 import warnings
 from pathlib import Path
-from re import Match
-from typing import IO, Any, Iterator
+from typing import TYPE_CHECKING
 
 from packaging.version import Version
 
-from pdm._types import RepositoryConfig
-from pdm.compat import Distribution, importlib_metadata
+from pdm.compat import importlib_metadata
+
+if TYPE_CHECKING:
+    from re import Match
+    from typing import IO, Any, Iterator
+
+    from unearth import Link
+
+    from pdm._types import RepositoryConfig
+    from pdm.compat import Distribution
 
 _egg_fragment_re = re.compile(r"(.*)[#&]egg=[^&]*")
 
@@ -87,7 +94,7 @@ def find_project_root(cwd: str = ".", max_depth: int = 5) -> str | None:
     return None
 
 
-def convert_hashes(hashes: dict[str, str]) -> dict[str, list[str]]:
+def convert_hashes(hashes: dict[Link, str]) -> dict[str, list[str]]:
     """Convert Pipfile.lock hash lines into InstallRequirement option format.
 
     The option format uses a str-list mapping. Keys are hash algorithms, and
@@ -271,23 +278,28 @@ def is_path_relative_to(path: str | Path, other: str | Path) -> bool:
     return True
 
 
-def get_venv_like_prefix(interpreter: str | Path) -> Path | None:
+def get_venv_like_prefix(interpreter: str | Path) -> tuple[Path | None, bool]:
     """Check if the given interpreter path is from a virtualenv,
-    and return the prefix if found.
+    and return two values: the root path and whether it's a conda env.
     """
     interpreter = Path(interpreter)
     prefix = interpreter.parent
     if prefix.joinpath("conda-meta").exists():
-        return prefix
+        return prefix, True
 
     prefix = prefix.parent
-    if prefix.joinpath("pyvenv.cfg").exists() or prefix.joinpath("conda-meta").exists():
-        return prefix
+    if prefix.joinpath("pyvenv.cfg").exists():
+        return prefix, False
+    if prefix.joinpath("conda-meta").exists():
+        return prefix, True
 
-    virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
+    virtual_env = os.getenv("VIRTUAL_ENV")
     if virtual_env and is_path_relative_to(interpreter, virtual_env):
-        return Path(virtual_env)
-    return None
+        return Path(virtual_env), False
+    virtual_env = os.getenv("CONDA_PREFIX")
+    if virtual_env and is_path_relative_to(interpreter, virtual_env):
+        return Path(virtual_env), True
+    return None, False
 
 
 def find_python_in_path(path: str | Path) -> Path | None:
